//
// Global stuff for the entire antsd
//
// $Id: antsd.hh,v 1.10 2001/03/11 20:41:15 jakob Exp $
//
//  Copyright (C) 2000 Jakob Oestergaard <jakob@unthought.net>
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later
//  version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#ifndef __JOBD_HH
#define __JOBD_HH

#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif
#include <netinet/in.h>
#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif

#include <string>

//
// Below are constants that have something to do with
// how the cluster status is maintained.
//

const time_t LOCAL_METRIC_UPDATE_INTERVAL = 10; // [s] - can be fairly high as this is triggered on demand too
const time_t CLUSTER_UPDATE_INTERVAL = 5; // [s]
const time_t JOBNET_UPDATE_INTERVAL = 10; // [s]
const time_t JOBNET_UPDATE_LIMITS_INTERVAL = 120; // [s]
const time_t CLUSTER_METRIC_REQUEST_INTERVAL = 5; // [s] - max interval for: if no best host found, request metrics from peers

const time_t JOBNET_RESUBSCRIBE_AFTER = 30; // [s]
const time_t JOBNET_UNSUBSCRIBE_AFTER = 90; // [s]

const time_t JOBNET_CALLBACK_MAXAGE = 10; // [s]
const time_t CLUSTER_PREALLOC_SLOT_LIFE = 5; // [s]
const time_t CLUSTER_SERIAL_CACHE_LIFE = 5;  // [s]

const time_t JOBNET_ALLOC_RETRY_DELAY = 100; // [ms]
const int RETRY_DELAY = 10; // [s] - for socket failures, should never be used

const int JOBCONN_FIRST_PORT = 30000;
const int JOBCONN_LAST_PORT = 40000;
const int JOBCONN_AUTH_TIMEOUT = 5; // [s]
const int EMERGENCY_KILL_TIMEOUT = 5; // [s]

//
// Below are some constants that are security and
// administration related
//

const int JOBNET_PORT_DECIMAL = 86;
const int IDENT_PORT_DECIMAL = 113;

const unsigned PROCESS_MIN_UID = 500;
const unsigned UNIX_MAX_LINES_PENDING = 50;

const std::string RANDOMFILE = "/dev/urandom";
const std::string LOCALSOCK = "/tmp/antsd";
const std::string CONFIGFILE = "/etc/antsd.conf";
const std::string HOSTFILE = "/etc/antsd.hosts";
const std::string PROCESS_SHELL = "/bin/sh";


//
// These constants should not be touched unless
// you know what you're doing...
//
const unsigned MAX_PACKET = 1000;
const unsigned ALIAS_LEN  = 32;
const unsigned USERNAME_LEN = 64;
const unsigned JOBNET_MAGIC = 3735927486UL;

const unsigned PROCESS_MESSAGE_LEN = 1024;
const unsigned JOBNET_ALLOC_RETRIES = 10;

//
// A few global declarations that fit in nowhere else...
//

extern int IDENT_PORT;
extern int JOBNET_PORT;

void log_info(const std::string&, unsigned = 0);  // level 0 for info, 1 for errors
void exit_fatal(const std::string&);

//
// recall routines return false if they should be removed from the recall
// list.  Return true otherwise.
// Args are:  method, interval [msec], retries, usage_specific_data
// Args for method are:  times_left, usage_specific_data
void add_recall_routine(bool(*)(int, unsigned), unsigned, unsigned, unsigned);

//
// Must have a way to remove recalls
//
void remove_recall_routine(bool(*)(int,unsigned), unsigned);

struct internalerror {
  internalerror(const std::string& s) : message(s) {}
  std::string message;
};

//
// If this variable is true, we must not accept new connections
// from remote clients, and we must tell other servers that we
// are unavailable (by sending invalidated statistics back)
//
extern bool shutdown_issued;

#endif
