//
// Process management
//
// $Id: processes.hh,v 1.7 2001/03/11 20:41:16 jakob Exp $
//
//  Copyright (C) 2000 Jakob Oestergaard <jakob@unthought.net>
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later
//  version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#ifndef __PROCESSES_HH
#define __PROCESSES_HH

#include "cluster.hh"
#include "jobnet.hh"

#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif
#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif

#include <string>
#include <map>

struct process {
  process() : pid(0), spawn_time(0), auth_ok(false) {}
  jobt_metrics get_jobt_metrics() const;
  std::string type;  // XXX: prev. set at spawn time - why not before ?
  int pid;
  address_t peer;
  time_t spawn_time;
  bool auth_ok;
};

struct runjob {
  std::string type;
  int isock;
  address_t peer;
  int serial;  
  int parent_fd;
  std::string username;
};

const unsigned subprocess_ok_msg = 0xEA5E0FF;

// remote_connection_fd -> process entry mapping
extern std::map<int,process> process_table;

extern int process_sock;

//
// Initial initialization
//
void initialize_processes();

//
// This should be called at regular intervals to clean
// up after dead processes etc.
//
void maintain_processes();

//
// This will insert the set of subprocess pipes in an fd_set
//
void insert_process_pipe_set(fd_set&, int&);

//
// This will read from the process pipes
//
void read_processes_pipe_set(const fd_set&);

//
// The stand-alone process handling rant<->job comms.
//
void subprocess_handler(runjob&);

//
// The subprocess spawn handler
//
unsigned startup_subprocess(const address_t&, unsigned, 
			    const std::string&, const std::string&);

#endif
