//
// Declarations for the local AF_UNIX socket handling
//
// $Id: sock.hh,v 1.7 2001/03/11 20:41:16 jakob Exp $
//
//  Copyright (C) 2000 Jakob Oestergaard <jakob@unthought.net>
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later
//  version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#ifndef __SOCK_HH
#define __SOCK_HH

#include "jobnet.hh"
#include "cluster.hh"

#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif
#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif
#include <sys/types.h>

extern int localsock;

//
// Attempt to setup local socket, return -1 on failure
//
void setup_sock();

//
// Read from the socket, handle incoming commands
//
void read_sock();

//
// Treat exceptions on the socket
//
void except_sock();

//
// This should be called at regular intervals to expire
// stuck local connections etc.
//
void maintain_local();

//
// Any local UNIX socket connections can be inserted
// into an fd_set using this call
//
void insert_local_readers_set(fd_set&, int&);
void insert_local_writers_set(fd_set&, int&);

//
// Read/write from/to local connections in set
//
void read_local_set(const fd_set&);
void write_local_set(const fd_set&);

//
// Notify a client of slot allocation failure
//
void notify_client_alloc_failure(const address_t&, unsigned);

//
// Notify a client of new metrics/limits
//
void notify_client_metrics_change(const address_t&, const std::string&,
				  const jobt_limits&, const jobt_metrics&);

#endif
